@@ -29,8 +29,7 @@
 
 #include "util/u_memory.h"
 #include "util/u_debug.h"
-
-#include "vl_winsys.h"
+#include "util/u_sampler.h"
 
 #include "vdpau_private.h"
 
@@ -41,8 +40,9 @@ PUBLIC VdpStatus
 vdp_imp_device_create_x11(Display *display, int screen, VdpDevice *device,
                           VdpGetProcAddress **get_proc_address)
 {
-   VdpStatus ret;
+   struct pipe_screen *pscreen;
    vlVdpDevice *dev = NULL;
+   VdpStatus ret;
 
    if (!(display && device && get_proc_address))
       return VDP_STATUS_INVALID_POINTER;
@@ -64,7 +64,8 @@ vdp_imp_device_create_x11(Display *display, int screen, VdpDevice *device,
       goto no_vscreen;
    }
 
-   dev->context = vl_video_create(dev->vscreen);
+   pscreen = dev->vscreen->pscreen;
+   dev->context = pscreen->context_create(pscreen, dev->vscreen);
    if (!dev->context) {
       ret = VDP_STATUS_RESOURCES;
       goto no_context;
@@ -76,10 +77,10 @@ vdp_imp_device_create_x11(Display *display, int screen, VdpDevice *device,
       goto no_handle;
    }
 
-   vl_compositor_init(&dev->compositor, dev->context->pipe);
+   vl_compositor_init(&dev->compositor, dev->context);
+   pipe_mutex_init(dev->mutex);
 
    *get_proc_address = &vlVdpGetProcAddress;
-   VDPAU_MSG(VDPAU_TRACE, "[VDPAU] Device created succesfully\n");
 
    return VDP_STATUS_OK;
 
@@ -105,8 +106,6 @@ vlVdpPresentationQueueTargetCreateX11(VdpDevice device, Drawable drawable,
    vlVdpPresentationQueueTarget *pqt;
    VdpStatus ret;
 
-   VDPAU_MSG(VDPAU_TRACE, "[VDPAU] Creating PresentationQueueTarget\n");
-
    if (!drawable)
       return VDP_STATUS_INVALID_HANDLE;
 
@@ -142,8 +141,6 @@ vlVdpPresentationQueueTargetDestroy(VdpPresentationQueueTarget presentation_queu
 {
    vlVdpPresentationQueueTarget *pqt;
 
-   VDPAU_MSG(VDPAU_TRACE, "[VDPAU] Destroying PresentationQueueTarget\n");
-
    pqt = vlGetDataHTAB(presentation_queue_target);
    if (!pqt)
       return VDP_STATUS_INVALID_HANDLE;
@@ -160,21 +157,19 @@ vlVdpPresentationQueueTargetDestroy(VdpPresentationQueueTarget presentation_queu
 VdpStatus
 vlVdpDeviceDestroy(VdpDevice device)
 {
-   VDPAU_MSG(VDPAU_TRACE, "[VDPAU] Destroying device\n");
-
    vlVdpDevice *dev = vlGetDataHTAB(device);
    if (!dev)
       return VDP_STATUS_INVALID_HANDLE;
-      
+
+   pipe_mutex_destroy(dev->mutex);
    vl_compositor_cleanup(&dev->compositor);
-   vl_video_destroy(dev->context);
+   dev->context->destroy(dev->context);
    vl_screen_destroy(dev->vscreen);
 
+   vlRemoveDataHTAB(device);
    FREE(dev);
    vlDestroyHTAB();
 
-   VDPAU_MSG(VDPAU_TRACE, "[VDPAU] Device destroyed successfully\n");
-
    return VDP_STATUS_OK;
 }
 
@@ -244,3 +239,72 @@ vlVdpGetErrorString (VdpStatus status)
    default: return "Unknown Error";
    }
 }
+
+void
+vlVdpDefaultSamplerViewTemplate(struct pipe_sampler_view *templ, struct pipe_resource *res)
+{
+   const struct util_format_description *desc;
+
+   memset(templ, 0, sizeof(*templ));
+   u_sampler_view_default_template(templ, res, res->format);
+
+   desc = util_format_description(res->format);
+   if (desc->swizzle[0] == UTIL_FORMAT_SWIZZLE_0)
+      templ->swizzle_r = PIPE_SWIZZLE_ONE;
+   if (desc->swizzle[1] == UTIL_FORMAT_SWIZZLE_0)
+      templ->swizzle_g = PIPE_SWIZZLE_ONE;
+   if (desc->swizzle[2] == UTIL_FORMAT_SWIZZLE_0)
+      templ->swizzle_b = PIPE_SWIZZLE_ONE;
+   if (desc->swizzle[3] == UTIL_FORMAT_SWIZZLE_0)
+      templ->swizzle_a = PIPE_SWIZZLE_ONE;
+}
+
+void
+vlVdpResolveDelayedRendering(vlVdpDevice *dev, struct pipe_surface *surface, struct u_rect *dirty_area)
+{
+   struct vl_compositor_state *cstate;
+   vlVdpOutputSurface *vlsurface;
+
+   assert(dev);
+
+   cstate = dev->delayed_rendering.cstate;
+   if (!cstate)
+      return;
+
+   vlsurface = vlGetDataHTAB(dev->delayed_rendering.surface);
+   if (!vlsurface)
+      return;
+
+   if (!surface) {
+      surface = vlsurface->surface;
+      dirty_area = &vlsurface->dirty_area;
+   }
+
+   vl_compositor_render(cstate, &dev->compositor, surface, dirty_area, true);
+
+   dev->delayed_rendering.surface = VDP_INVALID_HANDLE;
+   dev->delayed_rendering.cstate = NULL;
+
+   /* test if we need to create a new sampler for the just filled texture */
+   if (surface->texture != vlsurface->sampler_view->texture) {
+      struct pipe_resource *res = surface->texture;
+      struct pipe_sampler_view sv_templ;
+
+      vlVdpDefaultSamplerViewTemplate(&sv_templ, res);
+      pipe_sampler_view_reference(&vlsurface->sampler_view, NULL);
+      vlsurface->sampler_view = dev->context->create_sampler_view(dev->context, res, &sv_templ);
+   }
+
+   return;
+}
+
+void
+vlVdpSave4DelayedRendering(vlVdpDevice *dev, VdpOutputSurface surface, struct vl_compositor_state *cstate)
+{
+   assert(dev);
+
+   vlVdpResolveDelayedRendering(dev, NULL, NULL);
+
+   dev->delayed_rendering.surface = surface;
+   dev->delayed_rendering.cstate = cstate;
+}
